import argparse
import sys
import os

sys.path.insert(1, os.path.dirname(os.path.realpath(__file__)))

import random_word
import random_sentence

def main():
    random_word_class = random_word.random_word()
    random_sentence_class = random_sentence.random_sentence()

    # Create the parser
    my_parser = argparse.ArgumentParser(prog='wonderwords',
                                        description='Random word and sentence generation')

    # Add the arguments
    my_parser.add_argument('Mode',
                           metavar='mode',
                           type=str,
                           help='the mode of the generation: rw (random word), rl (random words list), rs (random sentence)')

    my_parser.add_argument('-n',
                           action='store_true',
                           help='include nouns (rw or rl)')

    my_parser.add_argument('-v',
                           action='store_true',
                           help='include verbs (rw or rl)')

    my_parser.add_argument('-a',
                           action='store_true',
                           help='include adjectives (rw or rl)')

    my_parser.add_argument('-min',
                           action='store',
                           type=int,
                           help='minimum word length (rw or rl)')

    my_parser.add_argument('-max',
                           action='store',
                           type=int,
                           help='maximum word length  (rw or rl)')

    my_parser.add_argument('-amount',
                           action='store',
                           type=int,
                           help='length of the random words list (rl)')

    my_parser.add_argument('-sep',
                           action='store',
                           type=str,
                           help='what you want words in the list to be separated by (rl)')

    my_parser.add_argument('-s',
                           action='store_true',
                           help='Add a space after the separator in the list generated by rl')

    my_parser.add_argument('-sw',
                           action='store',
                           type=str,
                           help='what letter you want the word to start with (rw)')

    my_parser.add_argument('-stype',
                           action='store',
                           type=str,
                           help='sentence type: bb (bare bone), ss (simple sentence), bba (bare bone with adjective), s (sentence)')

    # Execute parse_args()
    args = vars(my_parser.parse_args())

    # Now for the logic
    if args['Mode'] == 'rw':
        pos_options = {'n': 'noun', 'v': 'verb', 'a': 'adjective'}

        pos_str = ''

        for key, value in pos_options.items():
            if args[key]:
                pos_str += f" {value}"

        if args['min'] is not None:
            len_min = args['min']
        else:
            len_min = 0

        if args['max'] is not None:
            len_max = args['max']
        else:
            len_max = 0

        if args['sw'] is None:
            print(random_word_class.word(include_parts_of_speech=pos_str, word_min_length=len_min, word_max_length=len_max))
        else:
            if len(args['sw']) != 1:
                print('Please choose ONE letter for the -sw parameter')
                quit()
            else:
                print(random_word_class.starts_with(letter=args['sw'], include_parts_of_speech=pos_str, word_min_length=len_min, word_max_length=len_max))
    elif args['Mode'] == 'rl':
        if args['amount'] is not None:
            list_len = args['amount']
        else:
            list_len = 1

        pos_options = {'n': 'noun', 'v': 'verb', 'a': 'adjective'}

        pos_str = ''

        for key, value in pos_options.items():
            if args[key]:
                pos_str += f" {value}"

        if args['min'] is not None:
            len_min = args['min']
        else:
            len_min = 0

        if args['max'] is not None:
            len_max = args['max']
        else:
            len_max = 0

        if args['sep'] is None:
            print(random_word_class.words_list(amount=list_len, include_parts_of_speech=pos_str, word_min_length=len_min, word_max_length=len_max))
        else:
            words_list_raw = random_word_class.words_list(amount=list_len, include_parts_of_speech=pos_str, word_min_length=len_min, word_max_length=len_max)
            list_str = ''
            for word in words_list_raw:
                if word != words_list_raw[-1]:
                    if args['s']:
                        list_str += f"{word}{args['sep']} "
                    else:
                        list_str += f"{word}{args['sep']}"
                else:
                    list_str += word

            print(list_str)


    elif args['Mode'] == 'rs':
        sentence_type_options = ['bb', 'ss', 'bba', 's']

        if args['stype'] is None:
            sentence_type = sentence_type_options[3]
        else:
            if args['stype'] not in sentence_type_options:
                print('Please choose a valid sentence type (bb, ss, bba, s). Type "wonderwords -h" for help')
                quit()
            else:
                sentence_type = args['stype']

        if sentence_type == 'bb':
            print(random_sentence_class.bare_bone_sentence())
        elif sentence_type == 'ss':
            print(random_sentence_class.simple_sentence())
        elif sentence_type == 'bba':
            print(random_sentence_class.bare_bone_with_adjective())
        else:
            print(random_sentence_class.sentence())
    else:
        print(f"The mode '{args['Mode']}' is invalid\nPlease choose a valid mode or type 'wonderwords -h' for help")
        quit()

if __name__ == '__main__':
    main()



'''
MIT License

Copyright (c) 2020 mrmaxguns

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
'''

# Import the random module
import random


# Class random_word is the class that contains all the random word functions
class random_word:
    # Initialize some important variables
    def __init__(self):
        # Import importlib (will help gather the text files containing the words)
        try:
            import importlib.resources as pkg_resources
        except ImportError:
            # For older versions of python:
            import importlib_resources as pkg_resources

        # Read the files
        nouns = pkg_resources.open_text('wonderwords', 'nounlist.txt').readlines()
        adjectives = pkg_resources.open_text('wonderwords', 'adjectivelist.txt').readlines()
        verbs = pkg_resources.open_text('wonderwords', 'verblist.txt').readlines()

        # Strip newlines from all the files
        def strip_newline(file):
            words_newline_stripped = []
            for w in file:
                words_newline_stripped.append(w.rstrip())
            return words_newline_stripped

        # Create variables that contain lists of all the words
        self.noun = strip_newline(nouns)
        self.verb = strip_newline(verbs)
        self.adjective = strip_newline(adjectives)

    def check_for_errors(self, word_min_length, word_max_length):
        standard_error_message = '\nPlease visit our documentation for more info:\n' \
                                 'https://github.com/mrmaxguns/wonderwordsmodule/wiki/Wonderwords-Documentation'

        # Check for any mistakes in parameters
        if not isinstance(word_min_length, int):
            raise TypeError('word_min_length must be int' + standard_error_message)

        if not isinstance(word_max_length, int):
            raise TypeError('word_max_length must be int' + standard_error_message)

        if word_min_length > word_max_length and word_max_length != 0:
            raise Exception('word_min_length cannot be greater than word_max_length' + standard_error_message)

        if word_min_length < 0 or word_max_length < 0:
            raise Exception('word_max_length and/or word_min_length cannot be negative' + standard_error_message)

    # Word function chooses a random word from the lists
    def word(self, include_parts_of_speech='', word_min_length=0, word_max_length=0):
        self.check_for_errors(word_min_length, word_max_length)

        # create a function that filters words based on their length
        def filter_by_length(words_list, word_min_length_, word_max_length_):
            if word_min_length_ != 0:
                for i in words_list.copy():
                    if len(i) < word_min_length_:
                        words_list.remove(i)

            if word_max_length_ != 0:
                for i in words_list.copy():
                    if len(i) > word_max_length_:
                        words_list.remove(i)

        # List of words a random word will be picked from
        list_of_words_to_choose_from = []
        something_chosen = False

        # Check which parts of speech to include
        if 'noun' in include_parts_of_speech:
            filter_by_length(self.noun, word_min_length, word_max_length)
            list_of_words_to_choose_from.extend(self.noun)
            something_chosen = True
        if 'verb' in include_parts_of_speech:
            filter_by_length(self.verb, word_min_length, word_max_length)
            list_of_words_to_choose_from.extend(self.verb)
            something_chosen = True
        if 'adjective' in include_parts_of_speech:
            filter_by_length(self.adjective, word_min_length, word_max_length)
            list_of_words_to_choose_from.extend(self.adjective)
            something_chosen = True
        if not something_chosen:
            filter_by_length(self.noun, word_min_length, word_max_length)
            filter_by_length(self.verb, word_min_length, word_max_length)
            filter_by_length(self.adjective, word_min_length, word_max_length)
            list_of_words_to_choose_from = self.noun + self.verb + self.adjective

        # Return the random word
        return random.choice(list_of_words_to_choose_from)

    # Words_list function chooses a certain amount of random word and returns them as a list
    def words_list(self, amount, include_parts_of_speech='', word_min_length=0, word_max_length=0):
        self.check_for_errors(word_min_length, word_max_length)
        list_of_words = []

        # Loop through the amount of words passed as a parameter
        for every_word in range(amount):
            # Append a random word to the list
            list_of_words.append(self.word(include_parts_of_speech, word_min_length, word_max_length))

        # Return the list
        return list_of_words

    # Starts with chooses a random word from the words that start with a certain letter
    def starts_with(self, letter, include_parts_of_speech='', word_min_length=0, word_max_length=0):
        self.check_for_errors(word_min_length, word_max_length)

        # create a function that filters words based on their length
        def filter_by_length(words_list, word_min_length_, word_max_length_):
            if word_min_length_ != 0:
                for i in words_list.copy():
                    if len(i) < word_min_length_:
                        words_list.remove(i)

            if word_max_length_ != 0:
                for i in words_list.copy():
                    if len(i) > word_max_length_:
                        words_list.remove(i)

        list_of_words_to_choose_from = []
        something_chosen = False

        # Include certain parts of speech
        if 'noun' in include_parts_of_speech:
            filter_by_length(self.noun, word_min_length, word_max_length)
            list_of_words_to_choose_from.extend(self.noun)
            something_chosen = True
        if 'verb' in include_parts_of_speech:
            filter_by_length(self.verb, word_min_length, word_max_length)
            list_of_words_to_choose_from.extend(self.verb)
            something_chosen = True
        if 'adjective' in include_parts_of_speech:
            filter_by_length(self.adjective, word_min_length, word_max_length)
            list_of_words_to_choose_from.extend(self.adjective)
            something_chosen = True
        if not something_chosen:
            filter_by_length(self.noun, word_min_length, word_max_length)
            filter_by_length(self.verb, word_min_length, word_max_length)
            filter_by_length(self.adjective, word_min_length, word_max_length)
            list_of_words_to_choose_from = self.noun + self.verb + self.adjective

        list_of_words_that_start_with_letter = []

        # Choose only those words which start with the passed letter
        for i in list_of_words_to_choose_from:
            if i[0] == letter:
                list_of_words_that_start_with_letter.append(i)

        # Return the word
        return random.choice(list_of_words_that_start_with_letter)



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="wonderwords",
    version="1.1.9",
    author="Maxim Rebguns",
    author_email="mrmaxguns@gmail.com",
    include_package_data=True,
    description="A python package for random words and sentences in the english language (random word and sentence generator)",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/mrmaxguns/wonderwordsmodule",
    packages=setuptools.find_packages(),
    package_data={'wonderwords': ['*.txt']},
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    entry_points = {
        'console_scripts': [
            'wonderwords = wonderwords.cmdline:main'
        ]
    }
)



'''
MIT License

Copyright (c) 2020 mrmaxguns

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
'''

import random

# The main class containing all the data and functions
class random_sentence:
    def __init__(self):
        # Import importlib (will help gather resources)
        try:
            import importlib.resources as pkg_resources
        except ImportError:
            import importlib_resources as pkg_resources

        # Read the files
        nouns = pkg_resources.open_text('wonderwords', 'nounlist.txt').readlines()
        adjectives = pkg_resources.open_text('wonderwords', 'adjectivelist.txt').readlines()
        verbs = pkg_resources.open_text('wonderwords', 'verblist.txt').readlines()

        # Strip newlines
        def strip_newline(words):
            words_newline_stripped = []
            for w in words:
                words_newline_stripped.append(w.rstrip())
            return words_newline_stripped

        # Variables store lists of words
        self.noun = strip_newline(nouns)
        self.verb = strip_newline(verbs)
        self.adjective = strip_newline(adjectives)

    # Randomly generate bare bone sentences
    def bare_bone_sentence(self):
        the_noun = random.choice(self.noun)
        the_verb = random.choice(self.verb)

        # Check for exceptions in english
        if the_verb[-1] == 'h' and (the_verb[-2] == 's' or the_verb[-2] == 'c'):
            return 'The %s %ses.' % (the_noun, the_verb)

        elif the_verb[-1] == 'y':
            the_verb_list = list(the_verb)
            del the_verb_list[-1]
            the_new_verb = ''.join(the_verb_list)
            return 'The %s %sies.' % (the_noun, the_new_verb)

        elif the_verb[-1] == 's' and the_verb[-2] == 's':
            return 'The %s %ses.' % (the_noun, the_verb)

        else:
            return 'The %s %ss.' % (the_noun, the_verb)

    def simple_sentence(self):
        the_direct_object = random.choice(self.noun)

        the_bare_bone_sentence = self.bare_bone_sentence()
        the_bare_bone_list = list(the_bare_bone_sentence)
        del the_bare_bone_list[-1]
        the_simple_sentence = ''.join(the_bare_bone_list)
        return '%s %s.' % (the_simple_sentence, the_direct_object)

    def bare_bone_with_adjective(self):
        the_noun = random.choice(self.noun)
        the_verb = random.choice(self.verb)
        the_adjective = random.choice(self.adjective)

        # Check for exceptions in english
        if the_verb[-1] == 'h' and (the_verb[-2] == 's' or the_verb[-2] == 'c'):
            return 'The %s %s %ses.' % (the_adjective, the_noun, the_verb)

        elif the_verb[-1] == 'y':
            the_verb_list = list(the_verb)
            del the_verb_list[-1]
            the_new_verb = ''.join(the_verb_list)
            return 'The %s %s %sies.' % (the_adjective, the_noun, the_new_verb)

        elif the_verb[-1] == 's' and the_verb[-2] == 's':
            return 'The %s %s %ses.' % (the_adjective, the_noun, the_verb)

        else:
            return 'The %s %s %ss.' % (the_adjective, the_noun, the_verb)

    def sentence(self):
        the_noun = random.choice(self.noun)
        the_verb = random.choice(self.verb)
        the_adjective = random.choice(self.adjective)
        the_direct_object = random.choice(self.noun)

        # Check for exceptions in english
        if the_verb[-1] == 'h' and (the_verb[-2] == 's' or the_verb[-2] == 'c'):
            return 'The %s %s %ses %s.' % (the_adjective, the_noun, the_verb, the_direct_object)

        elif the_verb[-1] == 'y':
            the_verb_list = list(the_verb)
            del the_verb_list[-1]
            the_new_verb = ''.join(the_verb_list)
            return 'The %s %s %sies %s.' % (the_adjective, the_noun, the_new_verb, the_direct_object)

        elif the_verb[-1] == 's' and the_verb[-2] == 's':
            return 'The %s %s %ses %s.' % (the_adjective, the_noun, the_verb, the_direct_object)

        else:
            return 'The %s %s %ss %s.' % (the_adjective, the_noun, the_verb, the_direct_object)
